#!/usr/bin/python
# -*- coding: utf-8 -*-

# django-pyvows extensions
# https://github.com/rafaelcaricio/django-pyvows

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 Rafael Caricio rafael@caricio.com

from threading import current_thread

class SettingsTracker(object):

    def install(self):
        actual_import = __builtins__['__import__']
        if actual_import != self._import:
            self.real_import = actual_import
            __builtins__['__import__'] = self._import

    def _import(self, name, globals=None, locals=None, fromlist=[], level=-1):
        result = apply(self.real_import, (name, globals, locals, fromlist, level))
        fromlist = (fromlist or [])
        if name == 'django.conf' and 'settings' in fromlist:
            result.settings = VowsSettings(result.settings)
        elif name == 'django' and 'conf' in fromlist:
            result.conf.settings = VowsSettings(result.conf.settings)
        return result

class VowsSettings(object):

    def __init__(self, original_settings):
        self.original_settings = original_settings

    def __getattr__(self, attr_name):
        thread = current_thread()
        if hasattr(thread, 'settings'):
            if hasattr(thread.settings, attr_name):
                return getattr(thread.settings, attr_name)
        return getattr(self.original_settings, attr_name)

settings_tracker = SettingsTracker()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# django-pyvows extensions
# https://github.com/rafaelcaricio/django-pyvows

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 Rafael Caricio rafael@caricio.com

from pyvows import Vows, expect
from django.http import HttpResponse


class Url(object):
    def __init__(self, context, path):
        self.context = context
        self.path = path

@Vows.assertion
def to_be_mapped(topic):
    verify_url_is_mapped_to_method(topic)

@Vows.assertion
def to_match_view(topic, view):
    verify_url_is_mapped_to_method(topic, view, True)

@Vows.assertion
def to_be_http_response(topic):
    expect(topic).to_be_instance_of(HttpResponse)

def verify_url_is_mapped_to_method(topic, method=None, assert_method_as_well=False):
    assert isinstance(topic, Url), "Only django_pyvows.Url items can be verified for mapping"

    from django.conf import settings
    project_urls = settings.ROOT_URLCONF

    urlpatterns = __import__(project_urls).urls.urlpatterns

    found = False
    matches_method = False
    for urlpattern in urlpatterns:
        regex = urlpattern.regex
        pattern = regex.pattern 
        actual_method = urlpattern.callback

        if topic.path == pattern:
            found = True
        if method == actual_method:
            matches_method = True

    assert found, "Expected url(%s) to be mapped but it wasn't"
    if assert_method_as_well:
        assert matches_method, "Expected url(%s) to match method(%s), but it didn't"



#!/usr/bin/python
# -*- coding: utf-8 -*-

# django-pyvows extensions
# https://github.com/rafaelcaricio/django-pyvows

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 Rafael Caricio rafael@caricio.com

import os
import re
from threading import local, current_thread

from pyvows import Vows
from django.http import HttpRequest

from django_pyvows import http_helpers
from django_pyvows.assertions import Url, Model, Template
from django_pyvows.server import DjangoServer
from django_pyvows.settings_manager import settings_tracker

DEFAULT_PORT = 3331
DEFAULT_HOST = '127.0.0.1'

class DjangoContext(Vows.Context):

    @classmethod
    def start_environment(cls, settings_path):
        if not settings_path:
            raise RuntimeError('The settings_path argument is required.')
        os.environ['DJANGO_SETTINGS_MODULE'] = settings_path
        settings_tracker.install()

    def __init__(self, parent):
        super(DjangoContext, self).__init__(parent)
        self.ignore('get_settings', 'template', 'request', 'model', 'url', 'find_in_parent',
                'start_environment', 'port', 'host', 'get_url', 'get', 'post')

    @property
    def settings(self):
        thread = current_thread()
        if not hasattr(thread, 'settings'):
            thread.settings = local()
        return thread.settings

    def setup(self):
        DjangoContext.start_environment(self.get_settings())

    def get_settings(self):
        return os.environ.get('DJANGO_SETTINGS_MODULE', 'settings')

    def url(self, path):
        return Url(self, path)

    def template(self, template_name, context):
        return Template(template_name, context)

    def request(self, **kw):
        return HttpRequest(**kw)

    def model(self, model_class):
        return Model(self, model_class)

    def get(self, path):
        return http_helpers.get(self.get_url(path))

    def post(self, path, params):
        return http_helpers.post(self.get_url(path), params)

    def find_in_parent(self, attr_name):
        ctx = self.parent
        while ctx:
            if hasattr(ctx, attr_name):
                return getattr(ctx, attr_name)
            ctx = ctx.parent
        raise ValueError('Host could not be found in the context or any of its parents')

    def get_url(self, path):
        try:
            return self.find_in_parent('get_url')(path)
        except ValueError:
            return path


class DjangoHTTPContext(DjangoContext):

    def start_server(self, host=None, port=None):
        if not port: port = DEFAULT_PORT
        if not host: host = DEFAULT_HOST

        self.address = (host, port)
        self.server = DjangoServer(host, port)
        self.server.start(self.settings)

    def __init__(self, parent):
        super(DjangoHTTPContext, self).__init__(parent)
        self.ignore('start_server', 'settings')

    @property
    def host(self):
        if hasattr(self, 'address'):
            return self.address[0]
        return self.find_in_parent('host')

    @property
    def port(self):
        if hasattr(self, 'address'):
            return self.address[1]
        return self.find_in_parent('port')

    def get_url(self, path):
        if re.match('^https?:\/\/', path):
            return path
        return 'http://%s:%d%s' % (self.host, self.port, path)




#!/usr/bin/python
# -*- coding: utf-8 -*-

# django-pyvows extensions
# https://github.com/rafaelcaricio/django-pyvows

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 Rafael Caricio rafael@caricio.com

__version__ = (0, 4, 1)



import urllib2
from urllib import urlencode

import mimetypes
import mimetools
import itertools

class MultiPartForm(object):
    """Accumulate the data to be used when posting a form."""

    def __init__(self):
        self.form_fields = []
        self.files = []
        self.boundary = mimetools.choose_boundary()
        return

    def get_content_type(self):
        return 'multipart/form-data; boundary=%s' % self.boundary

    def add_field(self, name, value):
        """Add a simple field to the form data."""
        self.form_fields.append((name, value))
        return

    def add_file(self, fieldname, filename, fileHandle, mimetype=None):
        """Add a file to be uploaded."""
        body = fileHandle.read()
        if mimetype is None:
            mimetype = mimetypes.guess_type(filename)[0] or 'application/octet-stream'
        self.files.append((fieldname, filename, mimetype, body))
        return

    def __str__(self):
        """Return a string representing the form data, including attached files."""
        # Build a list of lists, each containing "lines" of the
        # request.  Each part is separated by a boundary string.
        # Once the list is built, return a string where each
        # line is separated by '\r\n'.
        parts = []
        part_boundary = '--' + self.boundary

        # Add the form fields
        parts.extend(
            [ part_boundary,
              'Content-Disposition: form-data; name="%s"' % name,
              '',
              value,
            ]
            for name, value in self.form_fields
            )

        # Add the files to upload
        parts.extend(
            [ part_boundary,
              'Content-Disposition: file; name="%s"; filename="%s"' % \
                 (field_name, filename),
              'Content-Type: %s' % content_type,
              '',
              body,
            ]
            for field_name, filename, content_type, body in self.files
            )

        # Flatten the list and add closing boundary marker,
        # then return CR+LF separated data
        flattened = list(itertools.chain(*parts))
        flattened.append('--' + self.boundary + '--')
        flattened.append('')
        return '\r\n'.join(flattened)


def get(url):
    response = urllib2.urlopen(url)
    return response

def post(url, data):
    form = MultiPartForm()

    for field, value in data.iteritems():
        if hasattr(value, "read"):
            form.add_file(field, value.name, value)
        else:
            form.add_field(field, str(value))

    request = urllib2.Request(url)
    body = str(form)
    request.add_header('Content-type', form.get_content_type())
    request.add_header('Content-length', len(body))
    request.add_data(body)

    try:
        response = urllib2.urlopen(request)
    except urllib2.HTTPError, err:
        response = err

    return response




#!/usr/bin/python
# -*- coding: utf-8 -*-

# django-pyvows extensions
# https://github.com/rafaelcaricio/django-pyvows

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 Rafael Caricio rafael@caricio.com

from lxml.etree import fromstring
from lxml.cssselect import CSSSelector

from pyvows import Vows, expect
from django.template.loader import render_to_string


class Template(object):
    def __init__(self, template_name, context):
        self.template_name = template_name
        self.context = context
        self.doc = None

    def load(self):
        if self.doc is None:
            self.doc = fromstring(render_to_string(self.template_name, self.context))
        return self.doc

    def select_element(self, selector):
        sel = CSSSelector(selector)
        return sel(self.load())

    def _to_contain(self, selector):
        return len(self.select_element(selector)) > 0

    def get_text(self, selector):
        return "".join((c.text for c in self.select_element(selector)))

    def __unicode__(self):
        return self.template_name

@Vows.assertion
def to_have_contents_of(topic, expected):
    expect(topic.content).to_be_like(expected)

@Vows.assertion
def to_contain(topic, selector):
    assert isinstance(topic, Template), "Only django_pyvows.Template items can be verified for mapping"
    assert topic._to_contain(selector), "Expected template(%s) to have an element(%s), but it don't have" % \
            (unicode(topic), selector)

@Vows.assertion
def not_to_contain(topic, selector):
    assert isinstance(topic, Template), "Only django_pyvows.Template items can be verified for mapping"
    assert not topic._to_contain(selector), "Expected template(%s) to not have an element(%s), but it have" % \
            (unicode(topic), selector)




#!/usr/bin/python
# -*- coding: utf-8 -*-

# django-pyvows extensions
# https://github.com/rafaelcaricio/django-pyvows

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 Rafael Caricio rafael@caricio.com


import sys
import urllib2
from threading import Thread, local, current_thread
from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler

from django.core.handlers.wsgi import WSGIHandler


class WSGIRequestHandler(BaseHTTPRequestHandler):
    """A request handler that implements WSGI dispatching."""

    @property
    def server_version(self):
        return 'pyVowsServer'

    def make_environ(self):
        if '?' in self.path:
            path_info, query = self.path.split('?', 1)
        else:
            path_info = self.path
            query = ''

        environ = {
            'wsgi.version':         (1, 0),
            'wsgi.url_scheme':      'http',
            'wsgi.input':           self.rfile,
            'wsgi.errors':          sys.stderr,
            'wsgi.multithread':     False,
            'wsgi.multiprocess':    False,
            'wsgi.run_once':        False,
            'SERVER_SOFTWARE':      self.server_version,
            'REQUEST_METHOD':       self.command,
            'SCRIPT_NAME':          '',
            'PATH_INFO':            urllib2.unquote(path_info),
            'QUERY_STRING':         query,
            'CONTENT_TYPE':         self.headers.get('Content-Type', ''),
            'CONTENT_LENGTH':       self.headers.get('Content-Length', ''),
            'REMOTE_ADDR':          self.client_address[0],
            'REMOTE_PORT':          self.client_address[1],
            'SERVER_NAME':          self.server.server_address[0],
            'SERVER_PORT':          str(self.server.server_address[1]),
            'SERVER_PROTOCOL':      self.request_version
        }

        for key, value in self.headers.items():
            key = 'HTTP_' + key.upper().replace('-', '_')
            if key not in ('HTTP_CONTENT_TYPE', 'HTTP_CONTENT_LENGTH'):
                environ[key] = value

        return environ

    def run_wsgi(self):
        app = self.server.app
        environ = self.make_environ()
        headers_set = []
        headers_sent = []

        def write(data):
            assert headers_set, 'write() before start_response'
            if not headers_sent:
                status, response_headers = headers_sent[:] = headers_set
                code, msg = status.split(None, 1)
                self.send_response(int(code), msg)
                header_keys = set()
                for key, value in response_headers:
                    self.send_header(key, value)
                    key = key.lower()
                    header_keys.add(key)
                if 'content-length' not in header_keys:
                    self.close_connection = True
                    self.send_header('Connection', 'close')
                if 'server' not in header_keys:
                    self.send_header('Server', self.version_string())
                if 'date' not in header_keys:
                    self.send_header('Date', self.date_time_string())
                self.end_headers()

            assert type(data) is str, 'applications must write bytes'
            self.wfile.write(data)
            self.wfile.flush()

        def start_response(status, response_headers, exc_info=None):
            headers_set[:] = [status, response_headers]
            return write

        application_iter = app(environ, start_response)
        try:
            for data in application_iter:
                write(data)
            write('')
        finally:
            if hasattr(application_iter, 'close'):
                application_iter.close()
            application_iter = None

    def handle_one_request(self):
        self.raw_requestline = self.rfile.readline()
        if self.parse_request():
            return self.run_wsgi()

    def log_request(self, *args, **kwargs):
        pass


class DjangoServer(HTTPServer, object):

    def __init__(self, host, port):
        HTTPServer.__init__(self, (host, int(port)), WSGIRequestHandler)
        self.app = WSGIHandler()

    def start(self, settings):
        self.server_activate()
        self.thr = Thread(target=self.make_response_thread, args=(getattr(settings, '__dict__', settings),))
        self.thr.daemon = True
        self.thr.start()

    def make_response_thread(self, settings):
        thread = current_thread()
        if not hasattr(thread, 'settings'):
            thread.settings = local()
            for key, value in settings.items():
                setattr(thread.settings, key, value)
        while True:
            self.handle_request()





#!/usr/bin/python
# -*- coding: utf-8 -*-

# django-pyvows extensions
# https://github.com/rafaelcaricio/django-pyvows

# Licensed under the MIT license:
# http://www.opensource.org/licenses/mit-license
# Copyright (c) 2011 Rafael Caricio rafael@caricio.com

from pyvows import Vows, expect


class Model(object):
    def __init__(self, context, model):
        from django.db import models

        self.context = context
        if isinstance(model, models.Model):
            model = model.__class__
        self.model = model

    def __call__(self, *args, **kwargs):
        return self.model(*args, **kwargs)

    @property
    def admin(self):
        from django.contrib import admin
        if self.model not in admin.site._registry:
            raise admin.sites.NotRegistered('The model %s is not registered' % self.model.__name__)
        return admin.site._registry[self.model]

@Vows.assertion
def to_be_in_admin(topic):
    from django.contrib import admin
    try:
        assert topic.admin
    except admin.sites.NotRegistered:
        assert False, "The model %s isn't registered in admin." % topic.model.__name__

@Vows.assertion
def to_have_field(topic, field_name, field_class=None, **kwargs):
    from django.db import models

    if isinstance(topic, Model):
        topic = topic.model

    if isinstance(topic, models.Model):
        topic = topic.__class__

    if not field_class:
        field_class = models.Field

    field = topic._meta.get_field(field_name)
    assert isinstance(field, field_class), "The '%s.%s' is not an instance of '%s'" % (topic.__name__, field_name, field_class.__name__)
    if kwargs:
        for attr, value in kwargs.items():
            field_value = getattr(field, attr)
            assert field_value == value, "The field option '%s' should be equal to '%s', but it's equal to '%s'" % (attr, value, field_value)

@Vows.assertion
def to_be_cruddable(topic, defaults={}):
    import django.db.models.fields as fields
    instance = __create_or_update_instance(topic, None, defaults)

    assert instance, "An instance could not be created for model %s" % topic.model.__name__

    retrieved = topic.model.objects.get(id=instance.id)
    assert retrieved.id == instance.id, "An instance could not be retrieved for model %s with id %d" % (topic.model.__name__, instance.id)

    for key, value in defaults.iteritems():
        assert value == getattr(retrieved, key), "The default specified value of '%s' should have been set in the '%s' property of the instance but it was not" % (value, key)

    updated = __create_or_update_instance(topic, retrieved, defaults)

    for field, value in topic.model._meta._field_cache:
        if field.__class__ == fields.AutoField:
            continue

        if field.name in defaults:
            continue

        assert getattr(updated, field.name) != getattr(instance, field.name), "The instance should have been updated but the field %s is the same in both the original instance and the updated one (%s)." % (field.name, getattr(updated, field.name))

    instance.delete()
    object_count = topic.model.objects.count()
    assert object_count == 0, "Object should have been deleted, but it wasn't (count: %d)" % object_count

def __create_or_update_instance(topic, instance, defaults):
    import django.db.models.fields as fields
    arguments = {}
    for field, value in topic.model._meta._field_cache:
        if field.__class__ == fields.AutoField:
            continue

        if field.name in defaults:
            arguments[field.name] = defaults[field.name]
            continue

        if field.__class__ == fields.CharField:
            __add_char_value_for(field, instance, arguments)

    if instance:
        for key, value in arguments.iteritems():
            setattr(instance, key, value)

        instance.save()
        return instance

    return topic.model.objects.create(**arguments)

def __add_char_value_for(field, instance, arguments):
    value = "monty python"
    if instance:
        value = getattr(instance, field.name) + '2'
    if field.max_length:
        if instance:
            value = value[:len(value) - 2] + '2'
        value = (value * field.max_length)[:field.max_length]
    arguments[field.name] = value
